@@ -20,6 +20,7 @@ import hashlib
 import shutil
 import threading
 import time
+import re
 import sys
 
 # standard logger for this module
@@ -161,7 +162,7 @@ class CacheEntry(object):
         self.inode = inode
         self.blockno = blockno
         self.last_access = 0
-        self.pos = 0
+        self.pos = self.fh.tell()
         self.size = os.fstat(self.fh.fileno()).st_size
 
     def read(self, size=None):
@@ -223,13 +224,14 @@ class CacheDict(OrderedDict):
         self.max_entries = max_entries
         self.size = 0
 
-    def remove(self, key):
+    def remove(self, key, unlink=True):
         '''Remove *key* from disk and cache, update size'''
 
         el = self.pop(key)
         el.close()
-        el.unlink()
         self.size -= el.size
+        if unlink:
+            el.unlink()
 
     def is_full(self):
         return (self.size > self.max_size
@@ -277,12 +279,30 @@ class BlockCache(object):
         self.to_upload = None
         self.to_remove = None
 
-        if not os.path.exists(self.path):
+        if os.path.exists(self.path):
+            self.load_cache()
+            log.info('Loaded %d entries from cache', len(self.cache))
+        else:
             os.mkdir(self.path)
 
         # Initialized fromt the outside to prevent cyclic dependency
         self.fs = None
 
+    def load_cache(self):
+        '''Initialize cache from disk'''
+
+        for filename in os.listdir(self.path):
+            match = re.match('^(\\d+)-(\\d+)$', filename)
+            if not match:
+                continue
+            inode = int(match.group(1))
+            blockno = int(match.group(2))
+
+            el = CacheEntry(inode, blockno,
+                            os.path.join(self.path, filename), mode='r+b')
+            self.cache[(inode, blockno)] = el
+            self.cache.size += el.size
+
     def __len__(self):
         '''Get number of objects in cache'''
         return len(self.cache)
@@ -297,11 +317,20 @@ class BlockCache(object):
             self.upload_threads.append(t)
 
         self.to_remove = Queue(1000)
-        for _ in range(10):
-            t = threading.Thread(target=self._removal_loop)
+        with self.backend_pool() as backend:
+            has_delete_multi = backend.has_delete_multi
+
+        if has_delete_multi:
+            t = threading.Thread(target=self._removal_loop_multi)
             t.daemon = True # interruption will do no permanent harm
             t.start()
             self.removal_threads.append(t)
+        else:
+            for _ in range(20):
+                t = threading.Thread(target=self._removal_loop_simple)
+                t.daemon = True # interruption will do no permanent harm
+                t.start()
+                self.removal_threads.append(t)
 
     def _lock_obj(self, obj_id, release_global=False):
         '''Acquire lock on *obj*id*'''
@@ -340,18 +369,24 @@ class BlockCache(object):
         else:
             self.mlock.release((inode, blockno), noerror=noerror)
 
-    def destroy(self):
+    def destroy(self, keep_cache=False):
         '''Clean up and stop worker threads
 
         This method should be called without the global lock held.
         '''
 
-        log.debug('Dropping cache...')
+        log.debug('Flushing cache...')
         try:
             with lock:
-                self.drop()
+                if keep_cache:
+                    self.flush() # releases global lock
+                    for el in self.cache.values():
+                        assert not el.dirty
+                        el.close()
+                else:
+                    self.drop()
         except NoWorkerThreads:
-            log.error('Unable to drop cache, no upload threads left alive')
+            log.error('Unable to flush cache, no upload threads left alive')
 
         # Signal termination to worker threads. If some of them
         # terminated prematurely, continue gracefully.
@@ -392,7 +427,8 @@ class BlockCache(object):
         self.upload_threads = None
         self.removal_threads = None
 
-        os.rmdir(self.path)
+        if not keep_cache:
+            os.rmdir(self.path)
 
         log.debug('cleanup done.')
 
@@ -689,13 +725,13 @@ class BlockCache(object):
 
         return len(self.in_transit) > 0
 
-    def _removal_loop(self):
+    def _removal_loop_multi(self):
         '''Process removal queue'''
 
-        # This method may look more complicated than necessary, but
-        # it ensures that we read as many objects from the queue
-        # as we can without blocking, and then hand them over to
-        # the backend all at once.
+        # This method may look more complicated than necessary, but it ensures
+        # that we read as many objects from the queue as we can without
+        # blocking, and then hand them over to the backend all at once.
+
         ids = []
         while True:
             try:
@@ -704,7 +740,7 @@ class BlockCache(object):
             except QueueEmpty:
                 tmp = FlushSentinel
 
-            if tmp in (FlushSentinel,QuitSentinel) and ids:
+            if tmp in (FlushSentinel, QuitSentinel) and ids:
                 log.debug('removing: %s', ids)
                 try:
                     with self.backend_pool() as backend:
@@ -719,18 +755,26 @@ class BlockCache(object):
             if tmp is QuitSentinel:
                 break
 
+    def _removal_loop_simple(self):
+        '''Process removal queue'''
+
+        while True:
+            log.debug('reading from queue..')
+            id_ = self.to_remove.get()
+            if id_ is QuitSentinel:
+                break
+            with self.backend_pool() as backend:
+                try:
+                    backend.delete('s3ql_data_%d' % id_)
+                except NoSuchObject:
+                    log.warning('Backend lost object s3ql_data_%d' % id_)
+                    self.fs.failsafe = True
+
     @contextmanager
     def get(self, inode, blockno):
         """Get file handle for block `blockno` of `inode`
 
-        This method releases the global lock. The managed block, however,
-        is executed with the global lock acquired and MUST NOT release
-        it. This ensures that only one thread is accessing a given block
-        at a time.
-
-        Note: if `get` and `remove` are called concurrently, then it is
-        possible that a block that has been requested with `get` and
-        passed to `remove` for deletion will not be deleted.
+        This method releases the global lock.
         """
 
         #log.debug('started with %d, %d', inode, blockno)
@@ -1022,7 +1066,10 @@ class BlockCache(object):
         # break reference loop
         self.fs = None
 
-        if len(self.cache) == 0:
+        for el in self.cache.values():
+            if el.dirty:
+                break
+        else:
             return
 
         # Force execution of sys.excepthook (exceptions raised
