@@ -1,17 +1,16 @@
 '''
-lock.py - this file is part of S3QL (http://s3ql.googlecode.com)
+lock.py - this file is part of S3QL.
 
-Copyright (C) Nikolaus Rath <Nikolaus@rath.org>
+Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from .logging import logging, setup_logging, QuietError
-from .common import assert_fs_owner, PICKLE_PROTOCOL
+from .common import assert_fs_owner
 from .parse_args import ArgumentParser
 import llfuse
 import os
-import pickle
 import sys
 import textwrap
 
@@ -24,13 +23,12 @@ def parse_args(args):
         description=textwrap.dedent('''\
         Makes the given directory tree(s) immutable. No changes of any sort can
         be performed on the tree after that. Immutable entries can only be
-        deleted with s3qlrm. 
+        deleted with s3qlrm.
         '''))
 
     parser.add_debug()
     parser.add_quiet()
     parser.add_version()
-    parser.add_fatal_warnings()
 
     parser.add_argument('path', metavar='<path>', nargs='+',
                         help='Directories to make immutable.',
@@ -53,7 +51,7 @@ def main(args=None):
             raise QuietError('%s is a mount point.' % name)
         ctrlfile = assert_fs_owner(name)
         fstat = os.stat(name)
-        llfuse.setxattr(ctrlfile, 'lock', pickle.dumps((fstat.st_ino,), PICKLE_PROTOCOL))
+        llfuse.setxattr(ctrlfile, 'lock', ('%d' % fstat.st_ino).encode())
 
 if __name__ == '__main__':
     main(sys.argv[1:])
