@@ -1,17 +1,17 @@
 '''
-umount.py - this file is part of S3QL (http://s3ql.googlecode.com)
+umount.py - this file is part of S3QL.
 
-Copyright (C) 2008-2009 Nikolaus Rath <Nikolaus@rath.org>
+Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from .logging import logging, setup_logging
-from .common import CTRL_NAME, assert_s3ql_mountpoint
+from . import CTRL_NAME
+from .common import assert_s3ql_mountpoint, parse_literal
 from .parse_args import ArgumentParser
 import llfuse
 import os
-import pickle
 import subprocess
 import platform
 import sys
@@ -22,20 +22,19 @@ log = logging.getLogger(__name__)
 
 def parse_args(args):
     '''Parse command line
-     
-    This function writes to stdout/stderr and may call `system.exit()` instead 
+
+    This function writes to stdout/stderr and may call `system.exit()` instead
     of throwing an exception if it encounters errors.
     '''
 
     parser = ArgumentParser(
-        description=textwrap.dedent('''\ 
+        description=textwrap.dedent('''\
         Unmounts an S3QL file system. The command returns only after all data
         has been uploaded to the backend.'''))
 
     parser.add_debug()
     parser.add_quiet()
     parser.add_version()
-    parser.add_fatal_warnings()
 
     parser.add_argument("mountpoint", metavar='<mountpoint>',
                         type=(lambda x: x.rstrip('/')),
@@ -55,18 +54,18 @@ class UmountError(Exception):
 
     message = 'internal error'
     exitcode = 3
-    
+
     def __init__(self, mountpoint):
         super().__init__()
         self.mountpoint = mountpoint
-        
+
     def __str__(self):
         return self.message
 
 class UmountSubError(UmountError):
     message = 'Unmount subprocess failed.'
     exitcode = 2
-    
+
 class MountInUseError(UmountError):
     message = 'In use.'
     exitcode = 1
@@ -91,24 +90,32 @@ def get_cmdline(pid):
     and return None.
     '''
 
-    try:
-        output = subprocess.check_output(['ps', '-p', str(pid), '-o', 'args='],
-                                         universal_newlines=True).strip()
-    except subprocess.CalledProcessError:
-        log.warning('Unable to execute ps, assuming process %d has terminated.'
-                    % pid)
-        return None
+    if os.path.isdir('/proc'):
+        try:
+            with open('/proc/%d/cmdline' % pid, 'r') as cmd_file:
+                return cmd_file.read()
+
+        except FileNotFoundError:
+            return None
 
-    if output:
-        return output
     else:
-        return None
-    
+        try:
+            output = subprocess.check_output(['ps', '-p', str(pid), '-o', 'args='],
+                                             universal_newlines=True).strip()
+            if output:
+                return output
+
+        except subprocess.CalledProcessError:
+            log.warning('Error when executing ps, assuming process %d has terminated.'
+                        % pid)
+
+    return None
+
 def blocking_umount(mountpoint):
     '''Invoke fusermount and wait for daemon to terminate.'''
 
     with open('/dev/null', 'wb') as devnull:
-        if subprocess.call(['fuser', '-m', mountpoint], stdout=devnull, 
+        if subprocess.call(['fuser', '-m', mountpoint], stdout=devnull,
                            stderr=devnull) == 0:
             raise MountInUseError(mountpoint)
 
@@ -119,7 +126,7 @@ def blocking_umount(mountpoint):
 
     # Get pid
     log.debug('Trying to get pid')
-    pid = pickle.loads(llfuse.getxattr(ctrlfile, 's3ql_pid?'))
+    pid = parse_literal(llfuse.getxattr(ctrlfile, 's3ql_pid?'), int)
     log.debug('PID is %d', pid)
 
     # Get command line to make race conditions less-likely
@@ -175,7 +182,7 @@ def main(args=None):
     setup_logging(options)
 
     assert_s3ql_mountpoint(options.mountpoint)
-    
+
     try:
         if options.lazy:
             lazy_umount(options.mountpoint)
@@ -192,7 +199,7 @@ def main(args=None):
     except UmountError as err:
         print('%s: %s' % (options.mountpoint, err), file=sys.stderr)
         sys.exit(err.exitcode)
-    
+
     sys.exit(0)
 
 if __name__ == '__main__':
